@@ -21,15 +21,27 @@ import logging
 import time
 import urllib.parse
 from pathlib import Path
-from typing import Tuple, NewType, List
+from typing import List, NewType
 
 import requests
+import yaml
 
-FW_UPLOAD = '/v3/firmware-images'
+# APIs documentation:
+# https://developer.pelion.com/docs/device-management/current/service-api-references/service-api-documentation.html
+# Device directory -
+#   https://developer.pelion.com/docs/device-management-api/device-directory
+# Update Service -
+#   https://developer.pelion.com/docs/device-management-api/update-service
+
+DEVICES = 'v3/devices'
+DEVICE = 'v3/devices/{id}'
+
+FW_IMAGES = '/v3/firmware-images'
+FW_IMAGE = '/v3/firmware-images/{id}'
+FW_UPLOAD = FW_IMAGES
 FW_UPLOAD_JOBS = '/v3/firmware-images/upload-jobs'
 FW_UPLOAD_JOB = '/v3/firmware-images/upload-jobs/{id}'
 FW_UPLOAD_JOB_CHUNK = '/v3/firmware-images/upload-jobs/{id}/chunks'
-FW_IMAGE = '/v3/firmware-images/{id}'
 
 FW_MANIFESTS = '/v3/firmware-manifests'
 FW_MANIFEST = '/v3/firmware-manifests/{id}'
@@ -64,17 +76,34 @@ URL = NewType('URL', str)
 ID = NewType('ID', str)
 Phase = NewType('Phase', str)
 
-class UpdateServiceApi:
-    def __init__(self, host: URL, access_key: str):
+class PelionServiceApi:
+    def __init__(self, config_file: Path):
         """
         Create REST API provider for Update Service APIs
         :param host: Pelion service URL
         :param access_key: account access key
         """
-        self._host = host
-        self._default_headers = {
-            'Authorization': 'Bearer {}'.format(access_key)
-        }
+
+        config = {}
+        if config_file.is_file():
+            with config_file.open('rt') as fh:
+                config = yaml.safe_load(fh)
+                if 'access_key' not in config and 'api_key' in config:
+                    # replace api_key with access_key
+                    # for backward compatibility
+                    config['access_key'] = config.pop('api_key')
+
+        if 'host' in config and 'access_key' in config:
+            self._host = config['host']
+            self._default_headers = {
+                'Authorization': 'Bearer {}'.format(
+                    config['access_key']
+                )
+            }
+            return
+
+        raise AssertionError('Pelion service configurations '
+                             '(URL and access key) are not provided')
 
     def _url(self, api, **kwargs) -> str:
         """
@@ -130,25 +159,29 @@ class UpdateServiceApi:
             '=' * count, increments, progress)
         print(text, end='\n' if progress == 100 else '')
 
-    def fw_upload(self, fw_name: str, image: Path) -> Tuple[URL, str, ID]:
+    def fw_upload(self, fw_name: str, image: Path, encrypt: bool) \
+            -> dict:
         """
         Upload FW image
         :param fw_name: update candidate image name as will appear on Pelion
                portal
         :param image: candidate FW image
-        :return: tuple consisting of uploaded image URL, short image URL and ID
+        :param encrypt: request to encrypt FW image
+        :return: uploaded image meta
         """
         fw_size = image.stat().st_size
 
         if fw_size < FW_UPLOAD_MAX_SMALL_SIZE:
-            fw_image_url, short_image_url, fw_image_id = \
-                self._upload_small_image(fw_name, image)
+            fw_meta = self._upload_small_image(
+                fw_name, image, encrypt
+            )
         else:
-            fw_image_url, short_image_url, fw_image_id = \
-                self._upload_big_image(fw_name, fw_size, image)
+            fw_meta = self._upload_big_image(
+                fw_name, fw_size, image, encrypt
+            )
 
-        LOG.info('Uploaded FW image %s', fw_image_url)
-        return fw_image_url, short_image_url, fw_image_id
+        LOG.info('Uploaded FW image %s', fw_meta['datafile'])
+        return fw_meta
 
     def _upload_image_chunk(self, chunk: bytes, job_id: ID):
         """
@@ -168,18 +201,24 @@ class UpdateServiceApi:
         )
         response.raise_for_status()
 
-    def _upload_big_image(self, fw_name: str, fw_size: int, image: Path) -> \
-            Tuple[URL, str, ID]:
+    def _upload_big_image(
+        self,
+        fw_name: str,
+        fw_size: int,
+        image: Path,
+        encrypt: bool
+    ) -> dict:
         """
         Helper function for uploading small FW image
         :param fw_name: fw name as appears on the portal
         :param image: candidate FW image
-        :return: tuple consisting of uploaded image URL, short image URL and ID
+        :param encrypt: Request to encrypt FW image
+        :return: uploaded image meta
         """
         job_id = None
         try:
             with image.open('rb') as fh:
-                job_id = self._create_upload_job(fw_name)
+                job_id = self._create_upload_job(fw_name, encrypt)
                 self._print_upload_progress(0, fw_size)
                 upload_counter = 0
                 while True:
@@ -198,13 +237,18 @@ class UpdateServiceApi:
             if job_id:
                 self._delete_upload_job(job_id)
 
-    def _upload_small_image(self, fw_name: str, image: Path) -> \
-            Tuple[URL, str, ID]:
+    def _upload_small_image(
+        self,
+        fw_name: str,
+        image: Path,
+        encrypt: bool
+    ) -> dict:
         """
         Helper function for uploading small FW image
         :param fw_name: fw name as appears on the portal
         :param image: candidate FW image
-        :return: tuple consisting of uploaded image URL, short image URL and ID
+        :param encrypt: Request to encrypt FW image
+        :return: uploaded image meta
         """
         with image.open('rb') as fh:
             response = requests.post(
@@ -214,20 +258,18 @@ class UpdateServiceApi:
                     'datafile': (image.name, fh),
                 },
                 data={
-                    'name': fw_name
+                    'name': fw_name,
+                    'datafile_encryption': encrypt
                 }
             )
             response.raise_for_status()
-            fw_image_url = response.json()['datafile']
-            short_image_url = response.json()['short_datafile']
-            fw_image_id = response.json()['id']
-            return fw_image_url, short_image_url, fw_image_id
+            return response.json()
 
-    def _get_fw_image_meta(self, job_id: ID) -> Tuple[URL, str, ID]:
+    def _get_fw_image_meta(self, job_id: ID) -> dict:
         """
         Helper function for extracting uploaded image URL and ID
         :param job_id: upload job ID
-        :return: tuple consisting of uploaded image URL, short image URL and ID
+        :return: uploaded image meta
         """
         response = requests.get(
             self._url(FW_UPLOAD_JOB, id=job_id),
@@ -238,9 +280,8 @@ class UpdateServiceApi:
             self._url(FW_IMAGE, id=image_id),
             headers=self._headers())
         response.raise_for_status()
-        url = response.json()['datafile']
-        short_url = response.json()['short_datafile']
-        return url, short_url, image_id
+
+        return response.json()
 
     def _delete_upload_job(self, job_id: ID):
         """
@@ -254,16 +295,20 @@ class UpdateServiceApi:
         response.raise_for_status()
         LOG.debug('FW upload job %s deleted', job_id)
 
-    def _create_upload_job(self, fw_name: str) -> ID:
+    def _create_upload_job(self, fw_name: str, encrypt: bool) -> ID:
         """
         Helper function for creating a firmware image upload job
         :param fw_name: fw name as appears on the portal
+        :param encrypt: Request to encrypt FW image
         :return: upload job ID
         """
         response = requests.post(
             self._url(FW_UPLOAD_JOBS),
             headers=self._headers(),
-            json={'name': fw_name}
+            json={
+                'name': fw_name,
+                'datafile_encryption': encrypt
+            }
         )
         response.raise_for_status()
         response_data = response.json()
@@ -368,10 +413,11 @@ class UpdateServiceApi:
         response.raise_for_status()
         LOG.info('Deleted campaign %s', campaign_id)
 
-    def campaign_stop(self, campaign_id: ID):
+    def campaign_stop(self, campaign_id: ID, timeout: int = 60):
         """
         Stop update campaign
         :param campaign_id: campaign ID
+        :param timeout: timeout in seconds to wait for campaign to stop
         """
         try:
             # check campaign phase and skip if it not active
@@ -385,8 +431,8 @@ class UpdateServiceApi:
             )
             response.raise_for_status()
             curr_phase = self.campaign_get(campaign_id)['phase']
-            # The campaign is stopping. wait a minute for it to finish
-            retries = 60
+            # The campaign is stopping. wait for it to stop
+            retries = timeout
             while curr_phase == 'stopping' and retries > 0:
                 time.sleep(1)
                 curr_phase = self.campaign_get(campaign_id)['phase']
@@ -467,7 +513,7 @@ class UpdateServiceApi:
         """
         Get metadata for devices participating in update campaign
 
-        Note: this function and assumes small number of devcises participating
+        Note: this function and assumes small number of devices participating
         in update campaign (as part of developer flow). Thus it does not
         handles pagination
         :param campaign_id: campaign ID
@@ -499,3 +545,77 @@ class UpdateServiceApi:
         :return: True if update campaign phase is starting or active
         """
         return phase in CAMPAIGN_NOT_STARTED_PHASES
+
+    def device_delete(self, device_id: ID):
+        """
+        Delete device with a developer certificate
+        :param device_id: device ID
+        """
+        response = requests.delete(
+            self._url(DEVICE, id=device_id),
+            headers=self._headers()
+        )
+        response.raise_for_status()
+        LOG.info('Deleted device %s', device_id)
+
+    def _get_objects(self, api_url: str, api_filter: str = '') -> dict:
+        """
+        Get list of objects for the account
+        :param api_filter: filter query string
+        :return: Dictionary with the list of objects
+        """
+        api_filter = '&filter={}'.format(api_filter) if api_filter else ''
+        url = '{}?limit=1000&include=total_count{}'.format(api_url, api_filter)
+        response = requests.get(
+            self._url(url),
+            headers=self._headers()
+        )
+        response.raise_for_status()
+        total_count = response.json()['total_count']
+        objects = response.json()['data']
+        LOG.debug('Got %d/%d objects', len(objects), total_count)
+
+        url = url + '&after={after}'
+        while len(objects) < total_count:
+            last_object_id = objects[-1]['id']
+            response = requests.get(
+                self._url(url, after=last_object_id),
+                headers=self._headers()
+            )
+            response.raise_for_status()
+            objects.extend(response.json()['data'])
+            LOG.debug('Got %d/%d objects', len(objects), total_count)
+
+        return objects
+
+    def get_devices(self, api_filter: str = '') -> dict:
+        """
+        Get list of devices for the account
+        :param api_filter: filter query string
+        :return: Dictionary with the list of devices
+        """
+        return self._get_objects(DEVICES, api_filter)
+
+    def get_fw_images(self, api_filter: str = '') -> dict:
+        """
+        Get list of firmware images for the account
+        :param api_filter: filter query string
+        :return: Dictionary with the list of firmware images
+        """
+        return self._get_objects(FW_IMAGES, api_filter)
+
+    def get_manifests(self, api_filter: str = '') -> dict:
+        """
+        Get list of manifests for the account
+        :param api_filter: filter query string
+        :return: Dictionary with the list of manifests
+        """
+        return self._get_objects(FW_MANIFESTS, api_filter)
+
+    def get_campaigns(self, api_filter: str = '') -> dict:
+        """
+        Get list of campaigns for the account
+        :param api_filter: filter query string
+        :return: Dictionary with the list of campaigns
+        """
+        return self._get_objects(FW_CAMPAIGNS, api_filter)
