@@ -99,7 +99,7 @@ class ManifestAsnCodecBase(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def set_delta_metadata(
+    def set_payload_metadata(
             self,
             installed_digest: bytes,
             installed_size: int,
@@ -129,35 +129,93 @@ class ManifestAsnCodecBase(abc.ABC):
 
         self.set_component_name(input_cfg.get('component', 'MAIN'))
 
-        vendor_id = self.encode_vendor_cfg(input_cfg)
+        vendor_id = self._encode_vendor_cfg(input_cfg)
 
-        self.encode_device_cfg(input_cfg, vendor_id)
+        self._encode_device_cfg(input_cfg, vendor_id)
 
-        return self._encode_payload_cfg(input_cfg)
+        installed_digest = self._encode_payload_cfg(input_cfg)
+
+        return installed_digest
 
     def _encode_payload_cfg(self, input_cfg) -> bytes:
+
         if 'payload' not in input_cfg:
             raise AssertionError('payload element not found')
+
+        self._encode_payload_url(input_cfg)
+
+        payload_format = self._encode_payload_format(input_cfg)
+
+        # calc 'file-path' (local file) digest and size for patch,
+        #  the local file is the generated patch
+        # for encrypted-raw, the local file is the plain image
+        local_digest, local_size = \
+            self._calc_file_digest_and_size(input_cfg)
+
+        if payload_format in (PayloadFormat.RAW, PayloadFormat.COMBINED):
+            payload_digest, payload_size = \
+                local_digest, local_size
+            installed_digest, installed_size, precursor_digest = \
+                local_digest, local_size, None
+
+        elif payload_format == PayloadFormat.PATCH:
+            payload_digest, payload_size = \
+                local_digest, local_size
+            installed_digest, installed_size, precursor_digest = \
+                self._get_delta_metadata(input_cfg)
+
+        elif payload_format in (PayloadFormat.ENCRYPTED_RAW,
+                                PayloadFormat.ENCRYPTED_COMBINED):
+            payload_digest, payload_size = \
+                self._get_encryption_metadata(input_cfg)
+            installed_digest, installed_size, precursor_digest = \
+                local_digest, local_size, None
+
+        else:
+            raise AssertionError('Unsupported payload format')
+
+        # set the downloaded payload fingerprint
+        self.set_payload_fingerprint(
+            digest=payload_digest,
+            size=int(payload_size)
+        )
+
+        if payload_format == PayloadFormat.RAW:
+            return payload_digest
+
+        # set the installed payload fingerprint
+        self.set_payload_metadata(
+            installed_digest=installed_digest,
+            installed_size=int(installed_size),
+            precursor_digest=precursor_digest)
+
+        return installed_digest
+
+    @staticmethod
+    def _calc_file_digest_and_size(input_cfg):
         if 'file-path' in input_cfg['payload']:
             file_path = input_cfg['payload']['file-path']
+            # calc digest and size of file-path
             with open(file_path, 'rb') as fh:
                 hash_ctx = hashlib.sha256()
                 buf = fh.read(READ_BLOCK_SIZE)
                 while buf:
                     hash_ctx.update(buf)
                     buf = fh.read(READ_BLOCK_SIZE)
-                installed_digest = hash_ctx.digest()
-                self.set_payload_fingerprint(
-                    digest=installed_digest,
-                    size=fh.tell()
-                )
+                file_digest = hash_ctx.digest()
+                file_size = fh.tell()
         else:
             raise AssertionError(
                 'payload:file-path must be provided')
+        return file_digest, file_size
+
+    def _encode_payload_url(self, input_cfg):
         if 'url' in input_cfg['payload']:
             self.set_payload_uri(input_cfg['payload']['url'])
         else:
             raise AssertionError('payload:url not found')
+
+    def _encode_payload_format(self, input_cfg):
         if 'format' in input_cfg['payload']:
             try:
                 payload_format = PayloadFormat(input_cfg['payload']['format'])
@@ -166,22 +224,39 @@ class ManifestAsnCodecBase(abc.ABC):
             self.set_payload_format(str(payload_format.value))
         else:
             raise AssertionError('payload-format element not found')
+        return payload_format
 
-        if payload_format == PayloadFormat.PATCH:
-            delta_file = Path(input_cfg['payload']['file-path'])
-            delta_config_file = delta_file.with_suffix('.yaml')
-            with delta_config_file.open('rb') as fh:
-                delta_cfg = yaml.safe_load(fh)
-            installed_digest = base64.b64decode(delta_cfg['installed-digest'])
-            self.set_delta_metadata(
-                installed_digest=installed_digest,
-                installed_size=int(delta_cfg['installed-size']),
-                precursor_digest=base64.b64decode(
-                    delta_cfg['precursor-digest'])
-            )
-        return installed_digest
+    @staticmethod
+    def _get_delta_metadata(input_cfg):
+        delta_file = Path(input_cfg['payload']['file-path'])
+        delta_config_file = delta_file.with_suffix('.yaml')
+        with delta_config_file.open('rb') as fh:
+            delta_cfg = yaml.safe_load(fh)
+        installed_digest = base64.b64decode(delta_cfg['installed-digest'])
+        installed_size = int(delta_cfg['installed-size'])
+        precursor_digest = \
+            base64.b64decode(delta_cfg['precursor-digest'])
+        return installed_digest, installed_size, precursor_digest
 
-    def encode_device_cfg(self, input_cfg, vendor_id):
+    @staticmethod
+    def _get_encryption_metadata(input_cfg):
+        if 'encrypted' not in input_cfg['payload']:
+            raise AssertionError('payload:encrypted must be provided')
+        if 'digest' not in input_cfg['payload']['encrypted']:
+            raise AssertionError(
+                'payload:encrypted digest must be provided')
+        if 'size' not in input_cfg['payload']['encrypted']:
+            raise AssertionError(
+                'payload:encrypted size must be provided')
+
+        encrypted_digest = bytearray.fromhex(
+            input_cfg['payload']['encrypted']['digest']
+        )
+        encrypted_size = int(input_cfg['payload']['encrypted']['size'])
+
+        return encrypted_digest, encrypted_size
+
+    def _encode_device_cfg(self, input_cfg, vendor_id):
         if 'device' not in input_cfg:
             raise AssertionError('device element not found')
         if 'model-name' in input_cfg['device']:
@@ -194,7 +269,7 @@ class ManifestAsnCodecBase(abc.ABC):
         logger.info('Class-ID: %s', class_id.bytes.hex())
         self.set_class_id(class_id.bytes)
 
-    def encode_vendor_cfg(self, input_cfg):
+    def _encode_vendor_cfg(self, input_cfg):
         if 'vendor' not in input_cfg:
             raise AssertionError('invalid vendor element not found')
         if 'domain' in input_cfg['vendor']:
